#!/bin/ksh

#**************************************************************
# Author: Sebastien Denvil, Martial Mancip
# Contact: Sebastien.Denvil@ipsl.jussieu.fr Martial.Mancip@ipsl.jussieu.fr
# $Date$
# $Author$
# $Revision$
# IPSL (2006)
#  This software is governed by the CeCILL licence see libIGCM/libIGCM_CeCILL.LIC
# History:
# Modification:
#
#**************************************************************

#===================================
function IGCM_config_Initialize
{
    IGCM_debug_PushStack "IGCM_config_Initialize"

    # Debug Print :
    echo
    IGCM_debug_Print 1 "IGCM_config_Initialize :"
    echo

    # Test modipsl tree existence.
    IGCM_sys_TestDir ${MODIPSL}
    [ $? != 0 ] && IGCM_debug_Exit "IGCM_sys_TestDirArchive"
    IGCM_sys_TestDir ${libIGCM}
    [ $? != 0 ] && IGCM_debug_Exit "IGCM_sys_TestDirArchive"
    IGCM_sys_TestDir ${R_EXE}
    [ $? != 0 ] && IGCM_debug_Exit "IGCM_sys_TestDirArchive"
    IGCM_sys_TestDir ${SUBMIT_DIR}
    [ $? != 0 ] && IGCM_debug_Exit "IGCM_sys_TestDirArchive"

    #==================================

    typeset option auxprint CompatibilityTag
    # Read libIGCM compatibility version in config.card
    IGCM_card_DefineVariableFromOption ${SUBMIT_DIR}/config.card Compatibility libIGCM
    eval CompatibilityTag=${config_Compatibility_libIGCM} > /dev/null 2>&1	

    if [ ! "${CompatibilityTag}" = "${libIGCM_CurrentTag}" ] ; then
	IGCM_debug_Exit "config.card is not compatible with libIGCM version ${libIGCM_CurrentTag} see libIGCM FAQ http://wiki.ipsl.jussieu.fr/wiki_ipsl/IGCMG/libIGCM/DocUtilisateur/FAQ ."
    fi

    #==================================
    IGCM_card_DefineArrayFromSection ${SUBMIT_DIR}/config.card UserChoices

    for option in ${config_UserChoices[*]} ; do
        IGCM_card_DefineVariableFromOption ${SUBMIT_DIR}/config.card UserChoices ${option}
    done

    IGCM_debug_Print 1 "DefineArrayFromOption  : config_UserChoices"
    IGCM_debug_PrintVariables 3 config_UserChoices_JobName
    IGCM_debug_PrintVariables 3 config_UserChoices_LongName
    IGCM_debug_PrintVariables 3 config_UserChoices_TagName
    IGCM_debug_PrintVariables 3 config_UserChoices_CalendarType
    IGCM_debug_PrintVariables 3 config_UserChoices_DateBegin
    IGCM_debug_PrintVariables 3 config_UserChoices_DateEnd
    IGCM_debug_PrintVariables 3 config_UserChoices_PeriodLength

    #==================================

    IGCM_card_DefineArrayFromSection ${SUBMIT_DIR}/config.card ListOfComponents

    echo
    IGCM_debug_Print 1 "DefineArrayFromSection : ListOfComponents"
    IGCM_debug_Print 3 ${config_ListOfComponents[*]}
    echo

    NbComponents=${#config_ListOfComponents[*]}

    #==================================
    # Define principal executable

    IGCM_card_DefineArrayFromSection   ${SUBMIT_DIR}/config.card Executable
    IGCM_card_DefineVariableFromOption ${SUBMIT_DIR}/config.card Executable Name
    
    #==================================
    # Define Outputs Name
    IGCM_debug_Print 1 "Define Script_Output_Prefix and Exe_Output"
    eval Script_Output_Prefix=${config_UserChoices_Script_Output_Prefix:='Script_Output'}
    IGCM_debug_Print 3 "Script_Output_Prefix = ${Script_Output_Prefix}"
    eval Exe_Output=out_${config_Executable_Name}
    IGCM_debug_Print 3 "Exe_Output           = ${Exe_Output}"

    #==================================

    #===================================================================#
    # Prepare variables available for ${COMP}.card and ${COMP}.driver   #
    #             But available to any son functions                    #
    #===================================================================#

    # Convert yyyy-mm-dd date to gregorian yyyymmdd
    DateBegin=$( IGCM_date_ConvertFormatToGregorian ${config_UserChoices_DateBegin} )
    DateEnd=$( IGCM_date_ConvertFormatToGregorian ${config_UserChoices_DateEnd} )

    # Period Length In Days between DateBegin and DateEnd
    (( ExperienceLengthInDays=$( IGCM_date_DaysBetweenGregorianDate ${DateEnd} ${DateBegin} )  + 1 ))
    if [ ${ExperienceLengthInDays} -lt 0 ] ; then
	IGCM_debug_Print 1 "Problem with dates in config.card : ${DateEnd} < ${DateBegin} ! You must check that."
	IGCM_debug_Exit "IGCM_config_Initialize" " Wrong Dates."
	IGCM_debug_Verif_Exit 
    fi

    # Day and Year of Initial State (Given in julian format)
    InitDay=$(  expr $( IGCM_date_ConvertGregorianDateToJulian $DateBegin ) % 1000 )
    InitYear=$( expr $( IGCM_date_ConvertGregorianDateToJulian $DateBegin ) / 1000 )

    #==================================
    # Restarts : Gerneral rule or local for each component ?
    IGCM_card_DefineArrayFromSection ${SUBMIT_DIR}/config.card Restarts

    echo
    IGCM_debug_Print 1 "DefineArrayFromOption : config_Restarts"

    for option in ${config_Restarts[*]} ; do
	IGCM_card_DefineVariableFromOption ${SUBMIT_DIR}/config.card Restarts ${option}
	eval auxprint=\${config_Restarts_${option}}
	IGCM_debug_Print 3 "${option} : ${auxprint}"
    done

    #==================================
    # Post : 
    IGCM_card_DefineArrayFromSection ${SUBMIT_DIR}/config.card Post

    echo
    IGCM_debug_Print 1 "DefineArrayFromOption : config_Post"

    for option in ${config_Post[*]} ; do
	IGCM_card_DefineVariableFromOption ${SUBMIT_DIR}/config.card Post ${option}
	eval auxprint=\${config_Post_${option}}
	IGCM_debug_Print 3 "${option} : ${auxprint}"
    done

    IGCM_debug_Print 1 "For now force RebuildFromArchive=true in libIGM_config"
    config_Post_RebuildFromArchive=true
    IGCM_debug_Print 3 "RebuildFromArchive : true"

    #================================================================#
    #                  Test and Prepare directories                  #
    #================================================================#

    # ==> 4 kinds of input files :
    #     1) R_INIT  : Initial State Files   (Etat0, carteveg)
    #     2) R_BC    : Boundary Conditions   (Forcages, lai)
    #     3) Parameters files (allready define through ${SUBMIT_DIR})
    #     4) Restarts files   (allready define in IGCM_config_Initialize)

    # Here we offer the possibility to redefine R_INIT, R_BC
    # and PeriodNb through config.card
    R_INIT=${config_UserChoices_R_INIT:=${R_IN}/INIT}
    echo
    IGCM_debug_Print 1 "(Re)Define R_INIT, R_BC and PeriodNb"
    IGCM_debug_Print 3 "R_INIT=${R_INIT}"
    R_BC=${config_UserChoices_R_BC:=${R_IN}/BC}
    IGCM_debug_Print 3  "R_BC=${R_BC}"
    PeriodNb=${config_UserChoices_PeriodNb:=${PeriodNb}}
    IGCM_debug_Print 3  "Loop in main Job with ${PeriodNb} period(s)"

    # Test Archive input/output.
    IGCM_sys_TestDirArchive ${ARCHIVE}
    [ $? != 0 ] && IGCM_debug_Exit "IGCM_sys_TestDirArchive"
    IGCM_sys_TestDirArchive ${R_INIT}
    [ $? != 0 ] && IGCM_debug_Exit "IGCM_sys_TestDirArchive"
    IGCM_sys_TestDirArchive ${R_BC}
    [ $? != 0 ] && IGCM_debug_Exit "IGCM_sys_TestDirArchive"

    #====================================================
    #R_SAVE : Job output directory
    R_SAVE=${R_OUT}/${config_UserChoices_TagName}/${config_UserChoices_JobName}

    if [ ! -f ${SUBMIT_DIR}/run.card ]; then 
	IGCM_sys_MkdirArchive ${R_SAVE}
    else
	#Test state of run in run.card
	IGCM_card_DefineVariableFromOption ${SUBMIT_DIR}/run.card Configuration PeriodState
	if ( [ ${run_Configuration_PeriodState} != "Start" ] && [ ${run_Configuration_PeriodState} != "Running" ] && [ ${run_Configuration_PeriodState} != "OnQueue" ] && [ ${run_Configuration_PeriodState} != "Continue" ] ) ; then
	    echo 
	    IGCM_debug_Print 1 "!!!! IGCM_config_Initialize Error PeriodState : " ${run_Configuration_PeriodState} "!!!!!!!!!!"
	    echo 
	    IGCM_debug_Exit "IGCM_config_Initialize Error PeriodState : " ${run_Configuration_PeriodState}
	    IGCM_debug_Verif_Exit
	fi
    fi
    IGCM_sys_TestDirArchive ${R_SAVE}
    [ $? != 0 ] && IGCM_debug_Exit "IGCM_sys_TestDirArchive"

    #====================================================
    #R_OUT_KSH : Storage place for job output
    #R_OUT_EXE : Storage place for binary used during simulation
    R_OUT_KSH=${R_SAVE}/Out
    R_OUT_EXE=${R_SAVE}/Exe

    IGCM_debug_PopStack "IGCM_config_Initialize"
}

#===================================
function IGCM_config_PeriodStart
{
    IGCM_debug_PushStack "IGCM_config_PeriodStart"

    echo
    IGCM_debug_Print 1 "IGCM_config_PeriodStart :"
    echo

    if [ ! -r ${SUBMIT_DIR}/run.card ]; then 

        #================================================#
        #         The file run.card doesn't exist        #
        #================================================#

	eval FirstInitialize=true

	#copy initial run.card
	IGCM_sys_Cp ${SUBMIT_DIR}/run.card.init ${SUBMIT_DIR}/run.card

	IGCM_date_GetYearMonth $DateBegin year month

	case ${config_UserChoices_PeriodLength} in
	    *Y|*y)
		PeriodLengthInYears=$( echo ${config_UserChoices_PeriodLength} | awk -F '[yY]' '{print $1}' ) 
	 	IGCM_debug_Print 1 "Number of years for PeriodLength : ${PeriodLengthInYears}"
	 	PeriodLengthInDays=0
	 	i=0
	 	until [ $i -ge $PeriodLengthInYears ] ; do 
	 	    (( PeriodLengthInDays = PeriodLengthInDays + $( IGCM_date_DaysInYear $(( year + i )) ) )) 
	 	    (( i=i+1 )) 
	 	done 
	 	;;
	    *M|*m)
	 	PeriodLengthInMonths=$( echo ${config_UserChoices_PeriodLength} | awk -F '[mM]' '{print $1}' ) 
	 	IGCM_debug_Print 1 "Number of months for PeriodLength : ${PeriodLengthInMonths}"
	 	typeset year1 
		PeriodLengthInDays=0
	 	year1=year
	 	i=0
	 	until [ $i -ge $PeriodLengthInMonths ] ; do 
	 	    if [ $(( month + i )) -lt 13 ] ; then 
	 	        (( PeriodLengthInDays  = PeriodLengthInDays + $( IGCM_date_DaysInMonth $year $(( month + i )) ) ))  
	 	    else 
 	                (( PeriodLengthInDays  = PeriodLengthInDays + $( IGCM_date_DaysInMonth $year $(( month + i - 12 )) ) ))  
	 	    fi 
 	            (( i=i+1 )) 
	 	done 
 	        ;;
	    *D|*d)  
	        PeriodLengthInDays=$( echo ${config_UserChoices_PeriodLength} | awk -F '[dD]' '{print $1}' ) ;;
	    *) 
		IGCM_debug_Exit "IGCM_config_PeriodStart " ${config_UserChoices_PeriodLength} " invalid period length : choose in *Y, *M, *D."
		IGCM_debug_Verif_Exit ;;
	esac

	eval PeriodDateBegin=${DateBegin} > /dev/null 2>&1
	PeriodDateEnd=$( IGCM_date_AddDaysToGregorianDate ${DateBegin} $( expr ${PeriodLengthInDays} - 1 ) )
	eval CumulPeriod=1 > /dev/null 2>&1

        #=================================================#
        #        Creation and write updated run.card      #
        #=================================================#

	#Correct run.card Configuration for this period
	IGCM_card_WriteOption ${SUBMIT_DIR}/run.card Configuration PeriodDateBegin ${PeriodDateBegin}
	IGCM_card_WriteOption ${SUBMIT_DIR}/run.card Configuration PeriodDateEnd ${PeriodDateEnd}
	IGCM_card_WriteOption ${SUBMIT_DIR}/run.card Configuration CumulPeriod ${CumulPeriod}
	IGCM_card_WriteOption ${SUBMIT_DIR}/run.card Configuration PeriodState "Running"
 
    else
        #================================================#
        #         The file run.card allready exist       #
        #================================================#

	eval FirstInitialize=false

	#Test state of run in run.card
	IGCM_card_DefineVariableFromOption ${SUBMIT_DIR}/run.card Configuration PeriodState
	if ( [ ${run_Configuration_PeriodState} != "Running" ] && [ ${run_Configuration_PeriodState} != "OnQueue" ] && [ ${run_Configuration_PeriodState} != "Continue" ] ) ; then
	    echo 
	    IGCM_debug_Print 1 "!!!!!! IGCM_config_PeriodStart Error PeriodState : " ${run_Configuration_PeriodState} "!!!!!!!!!!"
	    echo 
	    IGCM_debug_Exit "IGCM_config_PeriodStart Error PeriodState : " ${run_Configuration_PeriodState}
	    IGCM_debug_Verif_Exit
	fi

        #===================================#
        #        Read updated run.card      #
        #===================================#

	IGCM_card_DefineVariableFromOption ${SUBMIT_DIR}/run.card Configuration OldPrefix
	IGCM_card_DefineVariableFromOption ${SUBMIT_DIR}/run.card Configuration PeriodDateBegin
	IGCM_card_DefineVariableFromOption ${SUBMIT_DIR}/run.card Configuration PeriodDateEnd
	IGCM_card_DefineVariableFromOption ${SUBMIT_DIR}/run.card Configuration CumulPeriod

	PeriodDateBegin=$( IGCM_date_ConvertFormatToGregorian ${run_Configuration_PeriodDateBegin} )
	PeriodDateEnd=$( IGCM_date_ConvertFormatToGregorian ${run_Configuration_PeriodDateEnd} )
	eval CumulPeriod="${run_Configuration_CumulPeriod}" > /dev/null 2>&1

	if [ ${Period} = 1 ]; then
           # save last Job output
	    typeset LastPeriod Last_Script_Output_Name
	    (( LastPeriod=CumulPeriod-PeriodNb ))
	    Last_Script_Output_Name="${Script_Output_Prefix}_${config_UserChoices_JobName}.$( printf "%06d" ${LastPeriod} )"
 	    #eval ls -l ${SUBMIT_DIR}/${Last_Script_Output_Name}
 	    #eval IGCM_sys_Put_Out ${SUBMIT_DIR}/${Last_Script_Output_Name} ${R_OUT_KSH}/${Last_Script_Output_Name} > /dev/null 2>&1
	    IGCM_debug_Print 1 "Try to save previous ksh job output"
	    IGCM_sys_Cd ${SUBMIT_DIR}/
	    IGCM_sys_Put_Out ${Last_Script_Output_Name} ${R_OUT_KSH}/
	    IGCM_sys_Cd ${RUN_DIR}
	else
	    unset FileToBeDeleted
	fi

	IGCM_date_GetYearMonth $PeriodDateBegin year month

        # Determine number of day(s) in PeriodLength :
	case ${config_UserChoices_PeriodLength} in
	    *Y|*y)
		PeriodLengthInYears=$( echo ${config_UserChoices_PeriodLength} | awk -F '[yY]' '{print $1}' ) 
	 	IGCM_debug_Print 1 "Number of years for PeriodLength : ${PeriodLengthInYears}"
	 	PeriodLengthInDays=0
	 	i=0
	 	until [ $i -ge $PeriodLengthInYears ] ; do 
	 	    (( PeriodLengthInDays = PeriodLengthInDays + $( IGCM_date_DaysInYear $(( year + i )) ) )) 
	 	    (( i=i+1 )) 
	 	done 
	 	;;
	    *M|*m)
	 	PeriodLengthInMonths=$( echo ${config_UserChoices_PeriodLength} | awk -F '[mM]' '{print $1}' ) 
	 	IGCM_debug_Print 1 "Number of months for PeriodLength : ${PeriodLengthInMonths}"
	 	typeset year1 
		PeriodLengthInDays=0
	 	year1=year
	 	i=0
	 	until [ $i -ge $PeriodLengthInMonths ] ; do 
	 	    if [ $(( month + i )) -lt 13 ] ; then 
	 	        (( PeriodLengthInDays  = PeriodLengthInDays + $( IGCM_date_DaysInMonth $year $(( month + i )) ) ))  
	 	    else 
 	                (( PeriodLengthInDays  = PeriodLengthInDays + $( IGCM_date_DaysInMonth $year $(( month + i - 12 )) ) ))  
	 	    fi 
 	            (( i=i+1 )) 
	 	done 
 	        ;;
	    *D|*d)  
	        PeriodLengthInDays=$( echo ${config_UserChoices_PeriodLength} | awk -F '[dD]' '{print $1}' ) ;;
	    *) 
		IGCM_debug_Exit "IGCM_config_PeriodStart " ${config_UserChoices_PeriodLength} " invalid period length : choose in *Y, *M, *D."
		IGCM_debug_Verif_Exit ;;
	esac

	IGCM_card_WriteOption ${SUBMIT_DIR}/run.card Configuration PeriodState "Running"

    fi    
    OldPrefix=${config_UserChoices_JobName}_${PeriodDateEnd}
  
    #===================================================================#
    # Prepare variables available for ${COMP}.card and ${COMP}.driver   #
    #             But available to any son functions                    #
    #===================================================================#

    # Period Length In Days between DateBegin and DateCurrent (at end of period == PeriodDateEnd !)
    (( SimulationLengthInDays = $( IGCM_date_DaysBetweenGregorianDate ${PeriodDateEnd} ${DateBegin} ) + 1 ))

    # Debug Print :
    IGCM_debug_Print 1 "IGCM_config_PeriodStart : Before Execution"
    IGCM_debug_Print 3 "Year of simulation      : ${year}"
    IGCM_debug_Print 3 "Month of simulation     : ${month}"
    IGCM_debug_Print 3 "PeriodLengthInDays      : ${PeriodLengthInDays}"
    IGCM_debug_Print 3 "PeriodDateBegin         : ${PeriodDateBegin}"
    IGCM_debug_Print 3 "PeriodDateEnd           : ${PeriodDateEnd}"
    IGCM_debug_Print 3 "SimulationLengthInDays  : ${SimulationLengthInDays}"
    IGCM_debug_Print 3 "ExperienceLengthInDays  : ${ExperienceLengthInDays}"

    #================================================================#
    #         Prepare variables available for comp_finalyze          #
    #================================================================#

    # Period for save files
    eval DatesPeriod=${PeriodDateBegin}_${PeriodDateEnd} > /dev/null 2>&1

    # Prefix for save files of this period
    eval PREFIX=${config_UserChoices_JobName}_${DatesPeriod}  > /dev/null 2>&1

    # List of files that will be deleted in RUN_DIR after run
    [ -f stack ] && FileToBeDeleted[0]="stack"

    # Test if the same run as already been saved :
    if [ ${DRYRUN} -le 0 ] ; then
	if ( IGCM_sys_TestFileArchive ${R_OUT_KSH}/${PREFIX}_${Exe_Output} ) ; then
	    IGCM_debug_Exit "IGCM_config_PeriodStart" "RErun an old job."
	    IGCM_debug_Print 1 "Because of readonly permissions, you can't RErun a job when saved files"
	    IGCM_debug_Print 1 " are still in the ARCHIVE directory. You must deleted those files, or "
	    IGCM_debug_Print 1 " the whole ${R_SAVE} tree."
	    IGCM_debug_Verif_Exit
	fi
    fi
    IGCM_debug_PopStack "IGCM_config_PeriodStart"
}

#===================================
function IGCM_config_PeriodEnd
{
    IGCM_debug_PushStack "IGCM_config_PeriodEnd"

    echo
    IGCM_debug_Print 1 "IGCM_config_PeriodEnd :"
    echo

    #==================================#
    #         Save Job output          #
    #==================================#
    IGCM_sys_Put_Out ${Exe_Output} ${R_OUT_KSH}/${PREFIX}_${Exe_Output}
    eval FileToBeDeleted[${#FileToBeDeleted[@]}]="${Exe_Output}"

    typeset LS_comp LS_bin ExeDate ExeCpuLog ExeSize NextExeSize LastCompExeSize
    typeset comp i    
    typeset ExeNameIn ExeNameOut 

    #==================================#
    #        Get last Exe Size         #
    #==================================#

    (( i=0 ))
    if ( ${FirstInitialize} ) ; then
	run_Log_LastExeSize=""
	for comp in ${config_ListOfComponents[*]} ; do
	    run_Log_LastExeSize[$i]=0
	    (( i=i+1 ))
	done
    else
	IGCM_card_DefineArrayFromOption ${SUBMIT_DIR}/run.card Log LastExeSize
    fi

    #==================================#
    #         And Build ExeDate        #
    #==================================#

    # ExeDate = ATM_Jun_12_09:34-SRF_Jun_12_09:34-OCE_Jun_12_09:34-ICE_Jun_12_09:34-CPL_Jun_12_09:33
    # Would be nice to have next line but no way to format ls output (need to ls -l --time-style "+%Y-%m-%dT%H:%M")
    # ExeDate = ATM_2009-06-12T09:34+SRF_2009-06-12T09:34+OCE_2009-06-12T09:34+ICE_2009-06-12T09:34+CPL_2009-06-12T09:34
    ExeDate=""
    NextExeSize="( "
    (( i=0 ))

    for comp in ${config_ListOfComponents[*]} ; do

	IGCM_debug_Print 1 ${comp}

	eval ExeNameIn=\${config_Executable_${comp}[0]}
	eval ExeNameOut=\${config_Executable_${comp}[1]}
	if [ X${ExeNameIn} = X\"\" ] ; then
	    # If there is no exe file for this component
	    (( ExeSize=0 ))
	else
	    eval LS_bin=${R_EXE}/${ExeNameIn}
	    IGCM_sys_FileSize ${LS_bin} ExeSize

	    set +A LS_comp -- $( LC_TIME=en_US ls -l ${LS_bin} )
	    if [ X${ExeDate} = X ] ; then 
	        # First component exe date
		ExeDate=${comp}_${LS_comp[5]}_${LS_comp[6]}
	    else
		ExeDate=${ExeDate}-${comp}_${LS_comp[5]}_${LS_comp[6]}
	    fi
	    ExeDate=${ExeDate}_${LS_comp[7]}

	    eval FileToBeDeleted[${#FileToBeDeleted[@]}]=${ExeNameOut}
	fi

	if [ ${i} -eq 0 ] ; then
	    # First component
	    NextExeSize="( "${ExeSize}
	else
	    NextExeSize=${NextExeSize}", "${ExeSize}
	fi
	eval LastCompExeSize=${run_Log_LastExeSize[$i]}
	(( i=i+1 ))

	if [ ${ExeSize} -ne ${LastCompExeSize} ] ; then 
	    if ( ${FirstInitialize} ) ; then
		IGCM_debug_Print 1 "Put first ${ExeNameIn} in ${R_OUT_EXE} !"
	    else
		IGCM_debug_Print 1 "${ExeNameIn} has changed in ${R_EXE} !"
	    fi
	    eval IGCM_sys_Put_Out ${ExeNameOut} ${R_OUT_EXE}/${PREFIX}_${ExeNameIn}
	fi
    done

    if [ ${DRYRUN} -le 1 ] ; then
	typeset ExeCpuLog
	tail -1500 ${Exe_Output} > ${Exe_Output}_tail.txt
	ExeCpuLog=$( ${libIGCM}/libIGCM_sys/IGCM_add_out.awk ${Exe_Output}_tail.txt )
	RET=$?
	if [ $RET -eq 0 ] ; then
	    # ExeCpuLog variable contents 5 fields
	    echo "${CumulPeriod} ${PeriodDateBegin} ${PeriodDateEnd} ${ExeCpuLog} ${ExeDate}" |   \
	     awk '{printf("# %11d | %15s | %15s | %19s | %19s | %15.5f | %15.5f | %15.5f | %s\n", \
                              $1,$2,$3,$4,$5,$6,$7,$8,$9)}' >> ${SUBMIT_DIR}/run.card
	fi
	eval FileToBeDeleted[${#FileToBeDeleted[@]}]="${Exe_Output}_tail.txt"
    fi

    NextExeSize=${NextExeSize}" )"
    IGCM_card_WriteOption ${SUBMIT_DIR}/run.card Log LastExeSize "${NextExeSize}"

    # All was right ? no ? then we stop.
    IGCM_debug_Verif_Exit

    # If all was OK, we can delete all files not necessary for next Job
    IGCM_debug_Print 1 "Files that will be deleted before next period-run : "

    if [ ${DRYRUN} -le 2 ] ; then
	for f in ${FileToBeDeleted[@]} ; do ls -la $f ; rm -f $f ; done
    else
	echo ${FileToBeDeleted[@]}
    fi

    #=================================================#
    #         Determine next computed period          #
    #=================================================#

    PeriodDateBegin=$( IGCM_date_AddDaysToGregorianDate ${PeriodDateEnd} 1 )
    IGCM_date_GetYearMonth $PeriodDateBegin year month

    # Determine number of day(s) in PeriodLength :
    case ${config_UserChoices_PeriodLength} in
	*Y|*y)
	    PeriodLengthInYears=$( echo ${config_UserChoices_PeriodLength} | awk -F '[yY]' '{print $1}' ) 
	    IGCM_debug_Print 1 "Number of years for PeriodLength : ${PeriodLengthInYears}"
	    PeriodLengthInDays=0
	    i=0
	    until [ $i -ge $PeriodLengthInYears ] ; do 
	 	(( PeriodLengthInDays = PeriodLengthInDays + $( IGCM_date_DaysInYear $(( year + i )) ) )) 
	 	(( i=i+1 )) 
	    done 
	    ;;
	*M|*m)
	    PeriodLengthInMonths=$( echo ${config_UserChoices_PeriodLength} | awk -F '[mM]' '{print $1}' ) 
	    IGCM_debug_Print 1 "Number of months for PeriodLength : ${PeriodLengthInMonths}"
	    typeset year1 
	    PeriodLengthInDays=0
	    year1=year
	    i=0
	    until [ $i -ge $PeriodLengthInMonths ] ; do 
	 	if [ $(( month + i )) -lt 13 ] ; then 
	 	    (( PeriodLengthInDays  = PeriodLengthInDays + $( IGCM_date_DaysInMonth $year $(( month + i )) ) ))  
	 	else 
 	            (( PeriodLengthInDays  = PeriodLengthInDays + $( IGCM_date_DaysInMonth $year $(( month + i - 12 )) ) ))  
	 	fi 
 	        (( i=i+1 )) 
	    done 
 	    ;;
	*D|*d)  
	    PeriodLengthInDays=$( echo ${config_UserChoices_PeriodLength} | awk -F '[dD]' '{print $1}' ) ;;
    esac
    PeriodDateEnd=$( IGCM_date_AddDaysToGregorianDate ${PeriodDateBegin} $( expr ${PeriodLengthInDays} - 1 ) )

    # Debug Print :
    echo
    IGCM_debug_Print 1 "IGCM_config_PeriodEnd : Preparing Next Execution"
    IGCM_debug_Print 3 "PeriodDateBegin       : ${PeriodDateBegin}"
    IGCM_debug_Print 3 "PeriodDateEnd         : ${PeriodDateEnd}"
    IGCM_debug_Print 3 "PeriodLengthInDays    : ${PeriodLengthInDays}"

    PeriodDateBegin=$( IGCM_date_ConvertFormatToHuman ${PeriodDateBegin} )
    PeriodDateEnd=$( IGCM_date_ConvertFormatToHuman ${PeriodDateEnd} )

    (( CumulPeriod = CumulPeriod + 1 ))

    # Debug Print :
    echo
    IGCM_debug_Print 3 "PeriodDateBegin Human : ${PeriodDateBegin}"
    IGCM_debug_Print 3 "PeriodDateEnd Human   : ${PeriodDateEnd}"
    IGCM_debug_Print 3 "CumulPeriod           : ${CumulPeriod}"

    #=================================================#
    #             Write updated run.card              #
    #=================================================#

    IGCM_card_WriteOption ${SUBMIT_DIR}/run.card Configuration OldPrefix ${OldPrefix}
    IGCM_card_WriteOption ${SUBMIT_DIR}/run.card Configuration PeriodDateBegin ${PeriodDateBegin}
    IGCM_card_WriteOption ${SUBMIT_DIR}/run.card Configuration PeriodDateEnd ${PeriodDateEnd}
    IGCM_card_WriteOption ${SUBMIT_DIR}/run.card Configuration CumulPeriod ${CumulPeriod}

    IGCM_debug_PopStack "IGCM_config_PeriodEnd"
}

#===================================
function IGCM_config_Finalize
{
    IGCM_debug_PushStack "IGCM_config_Finalize"

    echo
    IGCM_debug_Print 1 "IGCM_config_Finalize :"
    echo

    # Supress Non Deleted Boundary files
    if [ ${DRYRUN} -le 2 ] ; then
	IGCM_comp_DelFixeBoundaryFiles
	ls -la
    fi

    if [ ${SimulationLengthInDays} -ge ${ExperienceLengthInDays} ] ; then

	#==========================#
        # End of entire simulation #
        #==========================#

	cat  << END_MAIL > job_atlas.mail
Dear ${LOGIN},

  Simulation ${config_UserChoices_JobName} is finished on supercomputer `hostname`.
  Job started : ${DateBegin}
  Job ended   : ${DateEnd}
  Ouput files are available in ${R_SAVE}
END_MAIL

	mailx -s "${config_UserChoices_JobName} completed" ${USER} < job_atlas.mail
	IGCM_card_WriteOption ${SUBMIT_DIR}/run.card Configuration PeriodState "Completed"
	IGCM_debug_Print 1 "Normal End of computation."
	if ( $DEBUG_debug ) ; then
	    echo
	    IGCM_debug_Print 1 "Your files on ${R_OUT} :"
	    IGCM_sys_Tree ${R_SAVE}
	fi
    else

	#=================#
        # Submit next job #
        #=================#

	IGCM_card_WriteOption ${SUBMIT_DIR}/run.card Configuration PeriodState "OnQueue"

        # Name of next Ksh Script output :
	eval Script_Output="${Script_Output_Prefix}_${config_UserChoices_JobName}.$( printf "%06d" ${CumulPeriod} )"

	IGCM_debug_Print 1 "Submit next job"
        # SUBMITTED NEXT JOB
	IGCM_sys_Cd ${SUBMIT_DIR}
	IGCM_sys_Qsub ${SUBMIT_DIR}/Job_${config_UserChoices_JobName}
    fi

    IGCM_debug_PopStack "IGCM_config_Finalize"
}

#===================================
